from _typeshed import Self
from typing import Any

from stripe import api_requestor as api_requestor, error as error
from stripe.stripe_object import StripeObject as StripeObject

class APIResource(StripeObject):
    @classmethod
    def retrieve(cls: type[Self], id, api_key: Any | None = ..., **params) -> Self: ...
    def refresh(self: Self) -> Self: ...
    @classmethod
    def class_url(cls) -> str: ...
    def instance_url(self) -> str: ...



from collections.abc import Iterator
from typing import Any

from stripe import api_requestor as api_requestor
from stripe.stripe_object import StripeObject as StripeObject

class ListObject(StripeObject):
    OBJECT_NAME: str
    def list(
        self, api_key: Any | None = ..., stripe_version: Any | None = ..., stripe_account: Any | None = ..., **params
    ) -> ListObject: ...
    def create(
        self,
        api_key: Any | None = ...,
        idempotency_key: str | None = ...,
        stripe_version: Any | None = ...,
        stripe_account: Any | None = ...,
        **params,
    ): ...
    def retrieve(
        self, id, api_key: Any | None = ..., stripe_version: Any | None = ..., stripe_account: Any | None = ..., **params
    ): ...
    def __getitem__(self, k): ...
    def __iter__(self): ...
    def __len__(self) -> int: ...
    def __reversed__(self): ...
    def auto_paging_iter(self) -> Iterator[Any]: ...
    @classmethod
    def empty_list(
        cls, api_key: Any | None = ..., stripe_version: Any | None = ..., stripe_account: Any | None = ...
    ) -> ListObject: ...
    @property
    def is_empty(self) -> bool: ...
    def next_page(
        self, api_key: Any | None = ..., stripe_version: Any | None = ..., stripe_account: Any | None = ..., **params
    ) -> ListObject: ...
    def previous_page(
        self, api_key: Any | None = ..., stripe_version: Any | None = ..., stripe_account: Any | None = ..., **params
    ) -> ListObject: ...



from stripe.api_resources.abstract import ListableAPIResource as ListableAPIResource
from stripe.api_resources.customer import Customer as Customer

class BitcoinReceiver(ListableAPIResource):
    OBJECT_NAME: str
    def instance_url(self): ...
    @classmethod
    def class_url(cls): ...



from typing import Any

class StripeResponseBase:
    code: Any
    headers: Any
    def __init__(self, code, headers) -> None: ...
    @property
    def idempotency_key(self): ...
    @property
    def request_id(self): ...

class StripeResponse(StripeResponseBase):
    body: Any
    data: Any
    def __init__(self, body, code, headers) -> None: ...

class StripeStreamResponse(StripeResponseBase):
    io: Any
    def __init__(self, io, code, headers) -> None: ...



from stripe import api_requestor as api_requestor
from stripe.api_resources.abstract import (
    CreateableAPIResource as CreateableAPIResource,
    DeletableAPIResource as DeletableAPIResource,
    ListableAPIResource as ListableAPIResource,
    SearchableAPIResource as SearchableAPIResource,
    UpdateableAPIResource as UpdateableAPIResource,
    custom_method as custom_method,
    nested_resource_class_methods as nested_resource_class_methods,
)

class Customer(CreateableAPIResource, DeletableAPIResource, ListableAPIResource, SearchableAPIResource, UpdateableAPIResource):
    OBJECT_NAME: str
    def delete_discount(self, **params) -> None: ...
    @classmethod
    def balance_transactions_url(cls, id, nested_id=...): ...
    @classmethod
    def balance_transactions_request(
        cls, method, url, api_key=..., idempotency_key=..., stripe_version=..., stripe_account=..., **params
    ): ...
    @classmethod
    def create_balance_transaction(cls, id, **params): ...
    @classmethod
    def retrieve_balance_transaction(cls, id, nested_id, **params): ...
    @classmethod
    def modify_balance_transaction(cls, id, nested_id, **params): ...
    @classmethod
    def list_balance_transactions(cls, id, **params): ...
    @classmethod
    def sources_url(cls, id, nested_id=...): ...
    @classmethod
    def sources_request(cls, method, url, api_key=..., idempotency_key=..., stripe_version=..., stripe_account=..., **params): ...
    @classmethod
    def create_source(cls, id, **params): ...
    @classmethod
    def retrieve_source(cls, id, nested_id, **params): ...
    @classmethod
    def modify_source(cls, id, nested_id, **params): ...
    @classmethod
    def delete_source(cls, id, nested_id, **params): ...
    @classmethod
    def list_sources(cls, id, **params): ...
    @classmethod
    def tax_ids_url(cls, id, nested_id=...): ...
    @classmethod
    def tax_ids_request(cls, method, url, api_key=..., idempotency_key=..., stripe_version=..., stripe_account=..., **params): ...
    @classmethod
    def create_tax_id(cls, id, **params): ...
    @classmethod
    def retrieve_tax_id(cls, id, nested_id, **params): ...
    @classmethod
    def delete_tax_id(cls, id, nested_id, **params): ...
    @classmethod
    def list_tax_ids(cls, id, **params): ...



from typing import Any

from stripe.api_resources import *
from stripe.oauth import OAuth as OAuth
from stripe.webhook import Webhook as Webhook, WebhookSignature as WebhookSignature

api_key: str | None
client_id: Any
api_base: str
connect_api_base: str
upload_api_base: str
api_version: Any
verify_ssl_certs: bool
proxy: Any
default_http_client: Any
app_info: Any
enable_telemetry: bool
max_network_retries: int
ca_bundle_path: Any
log: Any

def set_app_info(name, partner_id: Any | None = ..., url: Any | None = ..., version: Any | None = ...) -> None: ...



from _typeshed import Self
from typing import Any

from stripe import api_requestor as api_requestor
from stripe.api_resources.abstract import (
    CreateableAPIResource as CreateableAPIResource,
    DeletableAPIResource as DeletableAPIResource,
    ListableAPIResource as ListableAPIResource,
    SearchableAPIResource as SearchableAPIResource,
    UpdateableAPIResource as UpdateableAPIResource,
    custom_method as custom_method,
)

class Invoice(CreateableAPIResource, DeletableAPIResource, ListableAPIResource, SearchableAPIResource, UpdateableAPIResource):
    OBJECT_NAME: str
    def finalize_invoice(self: Self, idempotency_key: str | None = ..., **params) -> Self: ...
    def mark_uncollectible(self: Self, idempotency_key: str | None = ..., **params) -> Self: ...
    def pay(self: Self, idempotency_key: str | None = ..., **params) -> Self: ...
    def send_invoice(self: Self, idempotency_key: str | None = ..., **params) -> Self: ...
    def void_invoice(self: Self, idempotency_key: str | None = ..., **params) -> Self: ...
    @classmethod
    def upcoming(
        cls, api_key: Any | None = ..., stripe_version: Any | None = ..., stripe_account: Any | None = ..., **params
    ) -> Invoice: ...



from stripe.api_resources.abstract import (
    CreateableAPIResource as CreateableAPIResource,
    ListableAPIResource as ListableAPIResource,
    UpdateableAPIResource as UpdateableAPIResource,
    custom_method as custom_method,
    nested_resource_class_methods as nested_resource_class_methods,
)

class Transfer(CreateableAPIResource, ListableAPIResource, UpdateableAPIResource):
    OBJECT_NAME: str
    def cancel(self, idempotency_key: str | None = ..., **params): ...



from stripe.api_resources.abstract import (
    CreateableAPIResource as CreateableAPIResource,
    ListableAPIResource as ListableAPIResource,
    nested_resource_class_methods as nested_resource_class_methods,
)

class Session(CreateableAPIResource, ListableAPIResource):
    OBJECT_NAME: str



from _typeshed import Self
from typing import Any

from stripe import api_requestor as api_requestor
from stripe.api_resources.abstract.api_resource import APIResource as APIResource

class CreateableAPIResource(APIResource):
    @classmethod
    def create(
        cls: type[Self],
        api_key: Any | None = ...,
        idempotency_key: str | None = ...,
        stripe_version: Any | None = ...,
        stripe_account: Any | None = ...,
        **params,
    ) -> Self: ...



from typing import Any, overload
from typing_extensions import TypeAlias

from stripe.stripe_object import StripeObject
from stripe.stripe_response import StripeResponse

def utf8(value): ...
def log_debug(message, **params) -> None: ...
def log_info(message, **params) -> None: ...
def dashboard_link(request_id): ...
def logfmt(props): ...

class class_method_variant:
    class_method_name: Any
    def __init__(self, class_method_name) -> None: ...
    method: Any
    def __call__(self, method): ...
    def __get__(self, obj, objtype: Any | None = ...): ...

@overload
def populate_headers(idempotency_key: None) -> None: ...
@overload
def populate_headers(idempotency_key: str) -> dict[str, str]: ...

_RespType: TypeAlias = dict[Any, Any] | StripeObject | StripeResponse

# undocumented
@overload
def convert_to_stripe_object(
    resp: list[Any], api_key: Any | None = ..., stripe_version: Any | None = ..., stripe_account: Any | None = ...
) -> list[Any]: ...
@overload
def convert_to_stripe_object(
    resp: _RespType, api_key: Any | None = ..., stripe_version: Any | None = ..., stripe_account: Any | None = ...
) -> StripeObject: ...



from stripe.api_resources.abstract.api_resource import APIResource as APIResource
from stripe.api_resources.abstract.createable_api_resource import CreateableAPIResource as CreateableAPIResource
from stripe.api_resources.abstract.custom_method import custom_method as custom_method
from stripe.api_resources.abstract.deletable_api_resource import DeletableAPIResource as DeletableAPIResource
from stripe.api_resources.abstract.listable_api_resource import ListableAPIResource as ListableAPIResource
from stripe.api_resources.abstract.nested_resource_class_methods import (
    nested_resource_class_methods as nested_resource_class_methods,
)
from stripe.api_resources.abstract.searchable_api_resource import SearchableAPIResource as SearchableAPIResource
from stripe.api_resources.abstract.singleton_api_resource import SingletonAPIResource as SingletonAPIResource
from stripe.api_resources.abstract.updateable_api_resource import UpdateableAPIResource as UpdateableAPIResource
from stripe.api_resources.abstract.verify_mixin import VerifyMixin as VerifyMixin



from _typeshed import Self
from collections.abc import Iterator

from stripe.api_resources.abstract.api_resource import APIResource as APIResource
from stripe.api_resources.search_result_object import SearchResultObject

class SearchableAPIResource(APIResource):
    @classmethod
    def search(cls: type[Self], *args: str | None, **kwargs) -> SearchResultObject[Self]: ...
    @classmethod
    def search_auto_paging_iter(cls: type[Self], *args: str | None, **kwargs) -> Iterator[Self]: ...



from stripe import api_requestor as api_requestor
from stripe.api_resources.abstract import (
    CreateableAPIResource as CreateableAPIResource,
    ListableAPIResource as ListableAPIResource,
    SearchableAPIResource as SearchableAPIResource,
    UpdateableAPIResource as UpdateableAPIResource,
    custom_method as custom_method,
)

class Charge(CreateableAPIResource, ListableAPIResource, SearchableAPIResource, UpdateableAPIResource):
    OBJECT_NAME: str
    def capture(self, idempotency_key: str | None = ..., **params): ...
    def refund(self, idempotency_key: str | None = ..., **params): ...
    def update_dispute(self, idempotency_key: str | None = ..., **params): ...
    def close_dispute(self, idempotency_key: str | None = ..., **params): ...
    def mark_as_fraudulent(self, idempotency_key: str | None = ...): ...
    def mark_as_safe(self, idempotency_key: str | None = ...): ...
